#include <Arduino.h>
#include "max80_littlefs.h"
#include "max80_config.h"
#include <ArduinoJson.h> 
#include <FS.h> 
#include "LITTLEFS.h"
extern max80Config max80_config;
extern max80::littlefs max80_littlefs;
static const char TAG[] = __FILE__;
#define FORMAT_LITTLEFS_IF_FAILED true
int calcChecksum(uint8_t* address, uint16_t sizeToCalc)
{
  uint16_t checkSum = 0;
  
  for (uint16_t index = 0; index < sizeToCalc; index++)
  {
    checkSum += * ( ( (byte*) address ) + index);
  }

  return checkSum;
}
void listDir(fs::FS &fs, const char * dirname, uint8_t levels){
    Serial.printf("Listing directory: %s\r\n", dirname);

    File root = fs.open(dirname);
    if(!root){
        ESP_LOGE(TAG,"- failed to open directory");
        return;
    }
    if(!root.isDirectory()){
        ESP_LOGE(TAG," - not a directory");
        return;
    }

    File file = root.openNextFile();
    while(file){
        if(file.isDirectory()){
            ESP_LOGD(TAG,"  DIR : ");

#ifdef CONFIG_LITTLEFS_FOR_IDF_3_2
            Serial.println(file.name());
#else
            Serial.print(file.name());
            time_t t= file.getLastWrite();
            struct tm * tmstruct = localtime(&t);
            ESP_LOGD(TAG,"  LAST WRITE: %d-%02d-%02d %02d:%02d:%02d\n",(tmstruct->tm_year)+1900,( tmstruct->tm_mon)+1, tmstruct->tm_mday,tmstruct->tm_hour , tmstruct->tm_min, tmstruct->tm_sec);
#endif

            if(levels){
                listDir(fs, file.name(), levels -1);
            }
        } else {
            ESP_LOGD(TAG,"  FILE: %s ",file.name());
            ESP_LOGD(TAG,"  SIZE: %d",file.size());

#ifdef CONFIG_LITTLEFS_FOR_IDF_3_2
            Serial.println(file.size());
#else
            time_t t= file.getLastWrite();
            struct tm * tmstruct = localtime(&t);
            ESP_LOGD(TAG,"  LAST WRITE: %d-%02d-%02d %02d:%02d:%02d\n",(tmstruct->tm_year)+1900,( tmstruct->tm_mon)+1, tmstruct->tm_mday,tmstruct->tm_hour , tmstruct->tm_min, tmstruct->tm_sec);
#endif
        }
        file = root.openNextFile();
    }
}
namespace max80 
{

    void littlefs::setup(){
        //clean FS, for testing
        //SPIFFS.format();

        delay(1000);
        ESP_LOGD(TAG,"TAG");
        if(!LITTLEFS.begin(FORMAT_LITTLEFS_IF_FAILED)){
            ESP_LOGD(TAG,"LITTLEFS Mount Failed");
        }
        listDir(LITTLEFS, "/", 0);

        delay(1000);
        
    }
    bool littlefs::loadconfig(){
        File file = LITTLEFS.open(MAX80_CONFIG_FILNAME, "r");
        if (file)
        {
            max80_config.checksum = calcChecksum( (uint8_t*) &max80_config, sizeof(max80_config) - sizeof(max80_config.checksum) );
            
            file.read((uint8_t*) &max80_config, sizeof(max80_config));

            file.close();
            if ((String)max80_config.WiFi_Creds->wifi_ssid ==""){
                ESP_LOGD(TAG,"Write default to flash");
                memcpy(&max80_config.WiFi_Creds->wifi_ssid, "MAX80",5);
                memcpy(&max80_config.WiFi_Creds->wifi_pw, "max80!!!",8);
                max80_littlefs.saveconfig();
            }
            ESP_LOGD(TAG,"OK");
            return(true);
        }
        else
        {
            ESP_LOGD(TAG,"Write default to flash");
            memcpy(&max80_config.WiFi_Creds->wifi_ssid, "MAX80",5);
            memcpy(&max80_config.WiFi_Creds->wifi_pw, "max80!!!",8);
            max80_littlefs.saveconfig();
        }
        return(false);
    }
    bool littlefs::saveconfig(){
        File file = LITTLEFS.open(MAX80_CONFIG_FILNAME, "w",true);
        if (file)
        {
            max80_config.checksum = calcChecksum( (uint8_t*) &max80_config, sizeof(max80_config) - sizeof(max80_config.checksum) );
            
            file.write((uint8_t*) &max80_config, sizeof(max80_config));

            file.close();
            ESP_LOGD(TAG,"OK");
            return(true);
        }
        else
        {
            ESP_LOGD(TAG,"failed");
        }
        return(false);
 
    }
}

